@@ -32,6 +32,7 @@ from pyleoclim.utils.tsmodel import (
 )
 
 from statsmodels.tsa.arima_process import arma_generate_sample
+import matplotlib.pyplot as plt
 
 # a collection of useful functions
 
@@ -128,6 +129,7 @@ class TestUiSeriesSpectral:
 
         We will estimate the scaling slope of an ideal colored noise to make sure the result is reasonable.
         '''
+        np.random.seed(2333)  # fix the random seed to avoid random failures
         alpha = 1
         t, v = gen_colored_noise(nt=1000, alpha=alpha)
         ts = pyleo.Series(time=t, value=v)
@@ -419,13 +421,23 @@ class TestUiSeriesSummaryPlot:
         ts = pyleo.Series(time=t, value=v)
         psd = ts.spectral()
         scal = ts.wavelet()
+        period_label='Period'
+        psd_label='PSD'
+        time_label='Time'
+        value_label='Value'
         fig, ax = ts.summary_plot(
             psd=psd, scalogram=scal, figsize=[4, 5], title='Test',
-            period_label='Period', psd_label='PSD',
-            value_label='Value', time_label='Time',
+            period_label=period_label, psd_label=psd_label,
+            value_label=value_label, time_label=time_label,
             mute=True,
         )
+        
+        assert ax['scal'].properties()['ylabel'] == period_label, 'Period label is not being passed properly'
+        assert ax['psd'].properties()['xlabel'] == psd_label, 'PSD label is not being passed properly'
+        assert ax['scal'].properties()['xlabel'] == time_label, 'Time label is not being passed properly'
+        assert ax['ts'].properties()['ylabel'] == value_label, 'Value label is not being passed properly'
 
+        plt.close(fig)
 
 class TestUiSeriesCorrelation:
     ''' Test Series.correlation()
@@ -444,7 +456,7 @@ class TestUiSeriesCorrelation:
         ts2 = pyleo.Series(time=t, value=v2)
 
         corr_res = ts1.correlation(ts2, settings={'method': corr_method})
-        r = corr_res['r']
+        r = corr_res.r
         assert np.abs(r-1) < eps
 
     @pytest.mark.parametrize('corr_method', ['ttest', 'isopersistent', 'isospectral'])
@@ -461,17 +473,17 @@ class TestUiSeriesCorrelation:
         ts2 = pyleo.Series(time=t, value=v2)
 
         corr_res = ts1.correlation(ts2, settings={'method': corr_method})
-        r = corr_res['r']
+        r = corr_res.r
         assert np.abs(r-0) < eps
 
     @pytest.mark.parametrize('corr_method', ['ttest', 'isopersistent', 'isospectral'])
     def test_correlation_t2(self, corr_method, eps=0.1):
         ''' Test correlation between two series with inconsistent time axis
         '''
-        data = sio.loadmat(os.path.join(test_dirpath, '../../example_data/wtc_test_data_nino.mat'))
-        nino = data['nino'][:, 0]
-        air  = data['air'][:, 0]
-        t = data['datayear'][:, 0]
+        data = pd.read_csv('https://raw.githubusercontent.com/LinkedEarth/Pyleoclim_util/master/example_data/wtc_test_data_nino.csv')
+        nino = np.array(data['nino'])
+        air  = np.array(data['air'])
+        t = np.array(data['t'])
 
         # randomly delete 500 data pts
         n_del = 500
@@ -485,13 +497,13 @@ class TestUiSeriesCorrelation:
         ts1_evenly = pyleo.Series(time=t, value=air)
         ts2_evenly = pyleo.Series(time=t, value=nino)
         corr_res_evenly = ts1_evenly.correlation(ts2_evenly, settings={'method': corr_method})
-        r_evenly = corr_res_evenly['r']
+        r_evenly = corr_res_evenly.r
 
         ts1 = pyleo.Series(time=air_time_unevenly, value=air_value_unevenly)
         ts2 = pyleo.Series(time=nino_time_unevenly, value=nino_value_unevenly)
 
         corr_res = ts1.correlation(ts2, settings={'method': corr_method}, common_time_kwargs={'method': 'interp'})
-        r = corr_res['r']
+        r = corr_res.r
         assert np.abs(r-r_evenly) < eps
 
 class TestUiSeriesCausality:
@@ -680,7 +692,8 @@ class TestUISeriesWavelet():
     ''' Test the wavelet functionalities
     '''
 
-    @pytest.mark.parametrize('wave_method',['wwz','cwt'])
+    #@pytest.mark.parametrize('wave_method',['wwz','cwt'])
+    @pytest.mark.parametrize('wave_method',['wwz'])
     def test_wave_t0(self, wave_method):
         ''' Test Series.wavelet() with available methods using default arguments
         '''
@@ -689,7 +702,8 @@ class TestUISeriesWavelet():
         ts = pyleo.Series(time=t, value=v)
         scal = ts.wavelet(method=wave_method)
 
-    @pytest.mark.parametrize('wave_method',['wwz','cwt'])
+    #@pytest.mark.parametrize('wave_method',['wwz','cwt'])
+    @pytest.mark.parametrize('wave_method',['wwz'])
     def test_wave_t1(self,wave_method):
         '''Test Series.spectral() with WWZ/cwt with specified frequency vector passed via `settings`
         '''
@@ -700,25 +714,6 @@ class TestUISeriesWavelet():
         freq = np.linspace(1/500, 1/2, 20)
         scal = ts.wavelet(method=wave_method, settings={'freq': freq})
 
-class TestUISeriesSsa():
-    ''' Test the SSA functionalities
-    '''
-
-    def test_ssa_t0(self):
-        ''' Test Series.ssa() with available methods using default arguments
-        '''
-        t, v = gen_colored_noise(nt=500, alpha=1.0)
-        ts = pyleo.Series(time=t, value=v)
-        res = ts.ssa()
-
-    def test_ssa_t1(self):
-        '''Test Series.ssa() with var truncation
-        '''
-        alpha = 1
-        t, v = gen_colored_noise(nt=500, alpha=1.0)
-        ts = pyleo.Series(time=t, value=v)
-
-        res = ts.ssa(trunc='var')
 
 class TestUISeriesSsa():
     ''' Test the SSA functionalities
@@ -727,9 +722,13 @@ class TestUISeriesSsa():
     def test_ssa_t0(self):
         ''' Test Series.ssa() with available methods using default arguments
         '''
-        t, v = gen_colored_noise(nt=500, alpha=1.0)
-        ts = pyleo.Series(time=t, value=v)
-        res = ts.ssa()
+        nt = 500
+        t  = np.arange(nt)
+        cn = pyleo.gen_ts(model = 'colored_noise', t= t, alpha=1.0)
+
+        res = cn.ssa()
+        assert abs(res.pctvar.sum() - 100.0)<0.01
+        
 
     def test_ssa_t1(self):
         '''Test Series.ssa() with var truncation
@@ -748,7 +747,8 @@ class TestUISeriesSsa():
         t, v = gen_colored_noise(nt=500, alpha=1.0)
         ts = pyleo.Series(time=t, value=v)
 
-        res = ts.ssa(M=60, nMC=10, trunc='mc-ssa')
+        res = ts.ssa(M=60, nMC=10, trunc='mcssa')
+        res.screeplot(mute=True)
 
     def test_ssa_t3(self):
         '''Test Series.ssa() with Kaiser truncation
@@ -757,6 +757,14 @@ class TestUISeriesSsa():
         t, v = gen_colored_noise(nt=500, alpha=1.0)
         ts  = pyleo.Series(time=t, value=v)
         res = ts.ssa(trunc='kaiser')
+        
+    def test_ssa_t4(self):
+        '''Test Series.ssa() on Allen&Smith dataset
+        '''
+        df = pd.read_csv('https://raw.githubusercontent.com/LinkedEarth/Pyleoclim_util/Development/example_data/mratest.txt',delim_whitespace=True,names=['Total','Signal','Noise'])
+        mra = pyleo.Series(time=df.index, value=df['Total'], value_name='Allen&Smith test data', time_name='Time', time_unit='yr')
+        mraSsa = mra.ssa(nMC=10)
+        mraSsa.screeplot(mute=True)
 
 class TestUiSeriesPlot:
     '''Test for Series.plot()
@@ -776,9 +784,12 @@ class TestUiSeriesPlot:
 
         x_plot = line.get_xdata()
         y_plot = line.get_ydata()
+        
 
         assert_array_equal(t, x_plot)
         assert_array_equal(v, y_plot)
+        
+        plt.close(fig)
 
 class TestUiSeriesDistplot:
     '''Test for Series.distplot()'''
@@ -796,6 +807,8 @@ class TestUiSeriesDistplot:
         y_plot = line.get_ydata()
 
         assert max(x_plot) < max_axis
+        
+        plt.close(fig)
 
     def test_distplot_t1(self, vertical = True):
         t, v = gen_normal()
@@ -803,35 +816,50 @@ class TestUiSeriesDistplot:
         ts = pyleo.Series(time = t, value = v)
 
         fig, ax = ts.distplot(vertical=vertical, mute=True)
+        
+        plt.close(fig)
 
 class TestUiSeriesFilter:
     '''Test for Series.filter()'''
 
-    def test_filter_t0(self):
-        ''' Low-pass filtering with Butterworth
+    @pytest.mark.parametrize('method', ['butterworth', 'firwin','lanczos','savitzky-golay'])
+    def test_filter_t0(self, method):
+        ''' Low-pass filtering with Butterworth or FIR with window
         '''
         t = np.linspace(0, 1, 1000)
         sig1 = np.sin(2*np.pi*10*t)
         sig2 = np.sin(2*np.pi*20*t)
         sig = sig1 + sig2
         ts1 = pyleo.Series(time=t, value=sig1)
-        ts2 = pyleo.Series(time=t, value=sig2)
         ts = pyleo.Series(time=t, value=sig)
-        ts_lp = ts.filter(cutoff_freq=15)
+        ts_lp = ts.filter(cutoff_freq=15, method=method)
         val_diff = ts_lp.value - ts1.value
-        assert np.mean(val_diff**2) < 0.1
+        assert np.mean(val_diff**2) < 0.2
 
 
-    def test_filter_t1(self):
-        ''' Band-pass filtering with Butterworth
+    @pytest.mark.parametrize('method', ['butterworth', 'firwin'])
+    def test_filter_t1(self, method):
+        ''' Band-pass filtering with Butterworth or FIR with window
         '''
         t = np.linspace(0, 1, 1000)
         sig1 = np.sin(2*np.pi*10*t)
         sig2 = np.sin(2*np.pi*20*t)
         sig = sig1 + sig2
-        ts1 = pyleo.Series(time=t, value=sig1)
         ts2 = pyleo.Series(time=t, value=sig2)
         ts = pyleo.Series(time=t, value=sig)
-        ts_bp = ts.filter(cutoff_freq=[15, 25])
+        ts_bp = ts.filter(cutoff_freq=[15, 25], method=method)
         val_diff = ts_bp.value - ts2.value
         assert np.mean(val_diff**2) < 0.1
+        
+    # def test_filter_t2(self):
+    #     ''' Low-pass filtering with Lanczos
+    #     '''
+    #     t = np.linspace(0, 1, 1000)
+    #     sig1 = np.sin(2*np.pi*10*t)
+    #     sig2 = np.sin(2*np.pi*20*t)
+    #     sig = sig1 + sig2
+    #     ts1 = pyleo.Series(time=t, value=sig1)
+    #     ts = pyleo.Series(time=t, value=sig)
+    #     ts_lp = ts.filter(cutoff_freq=15, method = 'lanczos')
+    #     val_diff = ts_lp.value - ts1.value
+    #     assert np.mean(val_diff**2) < 0.1
\ No newline at end of file
