@@ -438,14 +438,21 @@ def enumerateTs(timeseries_list):
     available_y = []
     dataSetName =[]
     at =[]
-    for index,val in enumerate(timeseries_list):
-        for key, value in val.items():
-            if 'dataSetName' in key:
-                dataSetName.append(value)
-            if 'Data_variableName' in key:
-                available_y.append(value)
-            if 'archiveType' in key:
-                at.append(value)
+    for item in timeseries_list:
+        if 'dataSetName' in item.keys():
+            dataSetName.append(item['dataSetName'])
+        else:
+            dataSetName.append('NA')
+        if 'paleoData_variableName' in item.keys():
+            available_y.append(item['paleoData_variableName'])
+        elif 'chronData_variableName' in item.keys():
+            available_y.append(item['chronData_variableName'])
+        else:
+            available_y.append('NA')
+        if 'archiveType' in item.keys():
+            at.append(item['archiveType'])
+        else:
+            at.append('NA')
 
     for idx,val in enumerate(available_y):
         print(idx,': ',dataSetName[idx], ': ',at[idx],': ', val)
@@ -1498,13 +1505,13 @@ def getEnsemble(csv_dict, csvName):
         The matrix of Ensemble values
     """
     ensemble_dict=csv_dict[csvName]
+    ensembleValues=[]
     for val in ensemble_dict.keys():
-        number = ensemble_dict[val]["number"]
-        if type(number) is int:
+        if 'depth' in val:
             depth = ensemble_dict[val]["values"]
         else:
-            ensembleValues = ensemble_dict[val]["values"]
-            ensembleValues= np.transpose(np.array(ensembleValues))
+            ensembleValues.append(ensemble_dict[val]["values"])
+    ensembleValues= np.transpose(np.array(ensembleValues))
 
     return depth, ensembleValues
 
